package de.gematik.smartcard.g2.wrapper;

import de.gematik.smartcard.g2.wrapper.ApduLayerException.EnumApduLayerException;


/**
 * This class provides an exception handling for implementations of {@link IWrapper}.
 * <p>
 * <b>History:</b>
 * <ul>
 *   <li>2013-07-18: first edition
 * </ul>
 * 
 * @version 0.1
 * @author <a href="mailto:alfred.fiedler@gematik.de">Alfred Fiedler</a>
 */
public class WrapperException extends Exception {
  /** automatic generated number */
  private static final long serialVersionUID = 7306540806842016482L;

  /**
   * This class provides a collection of reasons why there is no corresponding response
   * APDU for a command APDU sent via {@link IApduLayer#sendAPDU(byte[])}.
   * <p>
   * <b>History:</b>
   * <ul>
   *   <li>2013-10-10: replace item <code>NoInstanceAvailable</code> by
   *                   <code>{@link #NoWrapperInstanceAvailable}</code>
   *   <li>2013-10-10: item <code>{@link #ObjectNotFound}</code> added
   *   <li>2013-07-18: first edition
   * </ul>
   * 
   * @version 0.2
   * @author <a href="mailto:alfred.fiedler@gematik.de">Alfred Fiedler</a>
   */
  public enum EnumWrapperException {
    /**
     * Object locator given in {@link IWrapper#getInformation(IApduLayer, byte[])}
     * is invalid due to
     * <ul>
     *   <li>byte array is not a valid TLV structure
     *   <li>TLV structure is in contradiction to specification
     *       (e.g. wrong tags, wrong tag sequence, etc.)
     * </ul>
     */
    InvalidObjectLocator,
    
    /**
     * An instance of a subclass of {@link IWrapper} could not be created.
     */
    NoWrapperInstanceAvailable,
    
    /**
     * An object pointed to by object locator could not be found in 'device under test'
     */
    ObjectNotFound,
    
    /**
     * This reason is intended for all other error situations. Instead of using
     * this reason a new enum SHALL be added to the list of reasons on request.
     */
    Other;
  } // end EnumWrapperException
  
  /** contains the reason why the exception is thrown */
  private final EnumWrapperException m_Reason;
  
  /** additional explanation in case {@link EnumApduLayerException#Other} is used */
  private final String m_Explanation;
  
  /**
   * @param reason causing the exception
   */
  public WrapperException(final EnumWrapperException reason) {
    m_Reason      = reason;
    m_Explanation = "";
  } // end constructor */
  
  /**
   * @param explanation describing why the exception is thrown
   */
  public WrapperException(final String explanation) {
    m_Reason      = EnumWrapperException.Other;
    m_Explanation = explanation;
  } // end constructor */
  
  /* (non-Javadoc)
   * @see java.lang.Throwable#getMessage()
   */
  @Override
  public String getMessage() {
    String result = m_Reason.toString();
    
    if(EnumWrapperException.Other == m_Reason) {
      result += ", " + m_Explanation;
    } // end if
    
    return result;
  } // end method */
} // end class